arch_layout <- function(mst_graph)  
{
	if (!is.igraph(mst_graph))
	{
		cat("Input has to be igraph object")
		return(NA)
	}
	if (!is.connected(mst_graph)) 
	{	
		cat("Cannot handle graph that has disjoint components\n")
		return(NA)
	}
	

	# get number of nodes, and binary adjacency matrix
	num_nodes = length(V(mst_graph));
	tmp = as.matrix(get.adjacency(mst_graph)) + t(as.matrix(get.adjacency(mst_graph))); 
	diag(tmp) = 0;
	adj =  array(as.double(tmp!=0), c(num_nodes,num_nodes));

	
	# make sure it is a tree, no circles
	tmp_adj = adj;
	diag(tmp_adj)=0;
	is_node_visited = array(0,c(num_nodes,1)); is_node_visited[1]=1;
	e = array(0,c(num_nodes,1)); e[1]=1;
	for (i in 1:num_nodes-1)
	{
		e_new = tmp_adj %*% e;
		tmp_adj[e!=0,]=0; tmp_adj[,e!=0]=0;
		e = e_new;
		if (any(e!=0 & is_node_visited==1))
		{
			cat("Cannot handle graphs with cycles\n");
			return(NA)
		}
		is_node_visited[e!=0,1]=1;
	}




	# find the shortest distance between each pairs of nodes
	shortest_hop = array(0,c(num_nodes,num_nodes));
	diag(adj) = 1;
	for (i in 1:num_nodes)
	{
		e = array(0,c(num_nodes,1));
		e[i]=1; 
		hop=0;
		while (any(shortest_hop[i,]==0))
		{
			e = array(as.double(adj %*% e > 0), c(num_nodes,1));
			hop = hop + 1;
			shortest_hop[i,t(e)>0 & shortest_hop[i,]==0]=hop;
		}
	}
	diag(adj)=0;
	diag(shortest_hop)=0;


	# find the longest path, which is the backbone arch
	tmp = which(shortest_hop==max(as.vector(shortest_hop)),arr.in=TRUE)[1,];
	backbone_start = min(tmp); backbone_end = max(tmp);
	back_bones = which(shortest_hop[backbone_start,]+shortest_hop[backbone_end,]==shortest_hop[backbone_start,backbone_end]);
	back_bones = back_bones[order(shortest_hop[backbone_start,back_bones])];


	# find sidechains and subsidechains
	side_chains = list();
	side_chain_roots = back_bones;
	counter = 1;
	while (counter<=length(side_chain_roots))
	{
		root_node = side_chain_roots[counter];
		first_neighbors = setdiff(which(adj[root_node,]!=0),side_chain_roots);
		if (length(first_neighbors)==0) {counter = counter + 1;	next;}
		for (i in 1:length(first_neighbors))
		{
			# find the side chain starting from root_node to first_neighbors[i] to as far as it can go
			subtree_nodes_through_this_neighbor = which(shortest_hop[root_node,]>shortest_hop[first_neighbors[i],]);
			end_node = subtree_nodes_through_this_neighbor[which(shortest_hop[root_node,subtree_nodes_through_this_neighbor]==max(shortest_hop[root_node,subtree_nodes_through_this_neighbor]))[1]];
			sub_backbone = which(shortest_hop[root_node,]+shortest_hop[end_node,]==shortest_hop[root_node,end_node]);
			sub_backbone = sub_backbone[order(shortest_hop[root_node,sub_backbone])];			

			side_chain_roots = c(side_chain_roots,sub_backbone[-1]);
			side_chains[[length(side_chains)+1]] = sub_backbone;
		}			
	}

#print(back_bones-1);
#if (length(side_chains)!=0) {for (i in 1:length(side_chains)) {print(side_chains[[i]]-1);}}

	# determine node location of the back_bones
	node_positions = array(0,c(2,num_nodes));
	position_assigned_flag = array(0,c(1,num_nodes));
	backbone_node_angles = 1:length(back_bones);
	backbone_node_angles = backbone_node_angles - mean(backbone_node_angles);
	backbone_node_angles = backbone_node_angles/max(backbone_node_angles)*(pi/90*25);
	node_positions[1,back_bones] = sin(backbone_node_angles);
	node_positions[2,back_bones] = -cos(backbone_node_angles);
	normalization_const = sqrt(as.vector(t(node_positions[,back_bones[1]]-node_positions[,back_bones[2]])%*%(node_positions[,back_bones[1]]-node_positions[,back_bones[2]])));
	node_positions = node_positions/normalization_const;
	position_assigned_flag[back_bones]=1;

	cat("generating layout for ", num_nodes, " nodes ... ",sum(position_assigned_flag), "\n");

if (length(side_chains)!=0)
{
	# determine the order of subbackbones to draw
	centerness = array(-1,c(1,length(side_chains)));
	for (k in 1:length(side_chains))
	{
		if (length(intersect(side_chains[[k]][1],back_bones))==0) {break;}
		centerness[k] = abs(shortest_hop[side_chains[[k]][1],back_bones[1]]-shortest_hop[side_chains[[k]][1],back_bones[length(back_bones)]]);
	}
	centerness = centerness[centerness!=-1];

	# determine node location for each node in the side chains
	side_chain_ordering = order(centerness);
	while(length(side_chain_ordering)<length(side_chains))
	{side_chain_ordering = c(side_chain_ordering,length(side_chain_ordering)+1);}


	for (i in side_chain_ordering)
	{
		for (j in 1:length(side_chains[[i]]))
		{
			if (position_assigned_flag[side_chains[[i]][j]]==1) {next;}
			new_node = side_chains[[i]][j];
			attaching_node = which(adj[new_node,]!=0 & position_assigned_flag==1);
			r = seq(from=0.3, to=0.9, by=0.04);
			theta = seq(from=0, to=2*pi, by=pi/180);
			potential_position_force = array(0,c(length(r),length(theta),2));
			for (m in 1:length(r))
			{
				for (n in 1:length(theta))
				{
					potential_position = node_positions[,attaching_node] + as.vector(r[m])*array(c(cos(theta[n]),sin(theta[n])),c(2,1));
					repel_vector = as.vector(potential_position) - node_positions[,position_assigned_flag==1];
					repel_force = rowSums(repel_vector/(array(1,c(2,1))%*%sqrt(colSums(repel_vector^2))^5));
					cos_alfa = (repel_force)%*%array(c(cos(theta[n]),sin(theta[n])),c(2,1))/sqrt(sum(repel_force^2));
					sin_alfa = sqrt(1-cos_alfa^2);

					potential_position_force[m,n,1]=sqrt(sum(repel_force^2))*sin_alfa;  # force perpendicular to the string
					potential_position_force[m,n,2]=sqrt(sum(repel_force^2))*cos_alfa;  # force along the sting
				}
			}

			best_for_each_layer = array(0,c(0,2));
			best_string_force_each_layer=array(0,c(0,1));
			for (m in 1:length(r))
			{
				n_s = which(potential_position_force[m, ,2]>=0);
				if (length(n_s)==0) {continue;}
				I = which(potential_position_force[m,n_s,1]==min(potential_position_force[m,n_s,1]))[1];
				best_for_each_layer = rbind(best_for_each_layer,c(m,n_s[I]));
				best_string_force_each_layer = rbind(best_string_force_each_layer, potential_position_force[m,n_s[I],2]);
			}
			I = which(best_string_force_each_layer==min(best_string_force_each_layer))[1];
			best_r = as.vector(r[best_for_each_layer[I,1]]);
			best_theta = as.vector(theta[best_for_each_layer[I,2]]);

			node_positions[,new_node] = node_positions[,attaching_node] + best_r*array(c(cos(best_theta),sin(best_theta)),c(2,1));
			position_assigned_flag[side_chains[[i]][j]]=1;
	
			cat("generating layout for ", num_nodes, " nodes ... ",sum(position_assigned_flag), "\n");

		}
	}

}

node_positions = node_positions - c(median(node_positions[1,]),median(node_positions[2,]));
node_positions[1,] = node_positions[1,]/max(abs(node_positions[1,]))*50;
node_positions[2,] = node_positions[2,]/max(abs(node_positions[2,]))*50;

t(node_positions)
}

